/*
 *
 * Interfaces.h - interfaces declarations
 *
 * Date:	10.6.2003
 * Update:	10.6. 2003
 * Author:	Jiri Formanek
 *
 */
#pragma once

class	IResourceManager;
class	ICamera;
typedef struct _CLIPVOLUME CLIPVOLUME;

#include "defs.h"

class IResource
{
public:
	virtual LPDIRECT3DDEVICE8		GetDevice() = 0;
	virtual void					SetID(DWORD dwID) = 0;
	virtual DWORD					GetID() = 0;
	virtual int						Restore() = 0;	
	virtual	int						ReleaseResource() = 0;
	virtual RES_TYPE				GetType() = 0;
	virtual	BOOL					operator==(IResource &pRes) = 0;

// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API IDisplay
{
public:
	virtual HRESULT					Init(HWND hWnd) = 0;
	virtual HRESULT					UpdateBackground() = 0;	
	virtual HRESULT					Present() = 0;
	virtual HRESULT					RestoreDisplay() = 0;
	virtual D3DXVECTOR2*			GetResolution() = 0;
	virtual ICamera *				GetCamera() = 0;

	virtual LPDIRECT3DDEVICE8		GetDevice() = 0;
	virtual LPDIRECT3D8				GetD3DObject() = 0;
	virtual IResourceManager*		GetResourceManager() = 0;
	virtual D3DFORMAT				GetTextureFormat() = 0;
	virtual float					GetCurrentFPS() = 0;

	virtual HRESULT					EnableLight(int iIndex, BOOL bEnable = TRUE)  = 0;
	virtual HRESULT					SetLight(int iIndex, D3DLIGHT8 * pLight)  = 0;

	// Fog methods
	virtual HRESULT	EnableFog(BOOL bEnable) = 0;
	virtual HRESULT	SetVertexFog(UINT uFogType, float fDensity, float fStart, float fEnd, D3DCOLOR Color) = 0;
	virtual HRESULT	SetPixelFog(UINT uFogType, float fDensity, float fStart, float fEnd, D3DCOLOR Color) = 0;

public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};


class DISPLAY_API ITexture : public IResource
{
public:
	virtual int						LoadTextureFromFile(LPCSTR szFileName)  = 0;
	virtual int						LoadTextureFromFileEx(LPCSTR szFileName, int iMipLevels, D3DFORMAT formatTextureFormat, D3DPOOL d3dPool = D3DPOOL_MANAGED) = 0;
	virtual LPDIRECT3DTEXTURE8		GetTexture() = 0;	
	virtual D3DFORMAT				GetFormat() = 0;
	virtual int						Width() = 0;
	virtual int						Height() = 0;
	virtual char*					GetTexturePath() = 0;
};

class DISPLAY_API I3DFont
{
public:	
	
	virtual HRESULT DrawText(const char* text, int x, int y, DWORD color) = 0;
	virtual HRESULT DrawTextEx(const char* text, int x, int y, DWORD colorTop, DWORD colorBottom, float scale) = 0;
	virtual HRESULT Load(BitmapFontLoadEnum filetype, char* fn) = 0;
	virtual HRESULT BeginDraw() = 0;
	virtual HRESULT EndDraw() = 0;
	virtual RECT GetTextRectEx(const char* text, int x, int y, float scale) = 0;
	virtual RECT GetTextRect(const char* text, int x, int y) = 0;

	/*virtual HRESULT	Draw(LPCSTR szText, int x, int y, D3DCOLOR dColor = -1) = 0;
	virtual HRESULT CreateFont(int iSize, int iWidth, LPCSTR szFamily, BOOL iBold, D3DCOLOR dColor) = 0;
	virtual LPD3DXFONT		GetFont() = 0;
	virtual LPDIRECT3DDEVICE8	GetDevice() = 0;
	virtual int		Restore() = 0;	
	virtual	int		ReleaseResource() = 0;*/

public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API IMesh : public IResource
{
public:
	virtual int						LoadMeshFromFile(LPCSTR szFileName, DWORD dwFlags) = 0;
	virtual int						SetSubsetColor(int iSubset, D3DCOLOR dwColor) = 0;
	virtual int						CreateSphere(float fRadius, UINT uSlices, UINT uStacks)  = 0;
	virtual int						CreateBox(float fWidth, float fHeight, float fDepth)  = 0;
	virtual int						CreateCylinder(float fRadius1, float fRadius2, float fLenght, UINT uSlices, UINT uStacks)  = 0;
	virtual int						CreateTeapot()  = 0;
	virtual int						CreateTorus(float fInnerRadius, float fOuterRadius, UINT uSides, UINT uRings)  = 0;
	virtual int						Draw() = 0;
	virtual int						Restore() = 0;
	virtual	int						ReleaseResource() = 0;
	virtual LPD3DXMESH				GetMesh() = 0;
	virtual LPDIRECT3DDEVICE8		GetDevice() = 0;
	virtual BOOL					IsVisible() = 0;
	virtual void					Visible(BOOL bVis = TRUE) = 0;
	virtual void					SetID(DWORD dwID) = 0;

	virtual void					Translate(float fX, float fY, float fZ) = 0;
	virtual void					RotateX(float fAngle) = 0;
	virtual void					RotateY(float fAngle) = 0;
	virtual void					RotateZ(float fAngle) = 0;
	virtual void					ScaleXYZ(float fFactor) = 0;
	virtual void					ScaleX(float fFactor) = 0;
	virtual void					ScaleY(float fFactor) = 0;
	virtual void					ScaleZ(float fFactor) = 0;
public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API IVertexBuffer
{
public:
	virtual int						Create(UINT uiLength, DWORD dwUsage, DWORD dwFVF, D3DPOOL dwPool) = 0;
	virtual int						Restore() = 0;	
	virtual	int						ReleaseResource() = 0;
	virtual LPDIRECT3DVERTEXBUFFER8	GetBuffer() = 0;
	virtual LPDIRECT3DDEVICE8		GetDevice() = 0;
	virtual void					SetID(DWORD dwID) = 0;

public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API IIndexBuffer
{
public:
	virtual int						Create(UINT uiLength, DWORD dwUsage, D3DFORMAT dwFormat, D3DPOOL dwPool) = 0;
	virtual int						Restore() = 0;	
	virtual	int						ReleaseResource() = 0;
	virtual LPDIRECT3DINDEXBUFFER8	GetBuffer() = 0;
	virtual LPDIRECT3DDEVICE8		GetDevice() = 0;
	virtual void					SetID(DWORD dwID) = 0;
public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API I3DObject
{
public:
	// standard methods
	virtual int						Init(	 
									LPCSTR szTexture, int iMipMap, D3DFORMAT formatTexture ,
									DWORD dwVBSize, 
									DWORD dwIBSize, 
									DWORD dwFlags) = 0; 
	virtual int						Draw(	D3DPRIMITIVETYPE Type,
									UINT MinIndex,
									UINT NumVertices,
									UINT StartIndex,
									UINT PrimitiveCount)  = 0;
	virtual int						Restore()  = 0;	
	virtual	int						ReleaseResource() = 0;
	virtual IVertexBuffer*			GetVB()	 = 0;
	virtual IIndexBuffer*			GetIB()	 = 0;
	virtual ITexture*				GetTexture() = 0;
	virtual LPDIRECT3DDEVICE8		GetDevice()	 = 0;
	virtual BOOL					IsVisible() = 0;
	virtual void					Visible(BOOL bVis = TRUE) = 0;
	virtual void					EnableTransformation(BOOL bEnable = TRUE) = 0;
	virtual BOOL					IsTransformationEnabled()  = 0;

	// transformations methods
	virtual void					Translate(float fX, float fY, float fZ) = 0;
	virtual void					RotateX(float fAngle) = 0;
	virtual void					RotateY(float fAngle) = 0;
	virtual void					RotateZ(float fAngle) = 0;
	virtual void					ScaleXYZ(float fFactor) = 0;
	virtual void					ScaleX(float fFactor) = 0;
	virtual void					ScaleY(float fFactor) = 0;
	virtual void					ScaleZ(float fFactor) = 0;	

public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API IResourceManager
{
public:
	virtual IResource*	AddResource(IResource *pRes, BOOL bUnique = FALSE) = 0;
	virtual IResource*	GetResource(DWORD dwID) = 0;

	virtual	HRESULT	AddVertexBuffer(IVertexBuffer * pBufferToAdd) = 0;
	virtual	HRESULT	AddIndexBuffer(IIndexBuffer * pBufferToAdd) = 0;	
	virtual HRESULT	AddSprite(ID3DXSprite * pSpriteToAdd) = 0;
	virtual	HRESULT ReleaseAll() = 0;
	virtual	HRESULT	RestoreAll() = 0;		
public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class DISPLAY_API ICamera
{
public:
	//
	// Inicialization
	virtual HRESULT				InitCamera(int iMinBound, int iMaxBound) = 0;
	// Update
	virtual HRESULT				ProcessCamera(float	fElapsedTime) = 0;	
	virtual	HRESULT				RestoreCamera() = 0;
	// Camera motion
	virtual void				ZoomIn() = 0;
	virtual void				ZoomOut() = 0;
	virtual void				RotateLeft() = 0;
	virtual void				RotateRight() = 0;
	virtual void				MoveLeft() = 0;
	virtual void				MoveRight() = 0;
	virtual void				MoveUp() = 0;
	virtual void				MoveDown() = 0;
	virtual void				IncreaseZAngle() = 0;
	virtual void				DecreaseZAngle() = 0;
	virtual void				SetCameraHeight(float fHeight) = 0;	
	//
	// Get methods
	virtual float				GetViewAngle() = 0; 
	virtual float				GetZoom()  = 0;
	virtual float				GetZAngle()  = 0;
	virtual D3DXVECTOR3*		GetCameraPos()  = 0;
	virtual D3DXVECTOR3*		GetEyePoint()  = 0;
	virtual D3DXMATRIX*			GetBillboardMatrix()  = 0;
	virtual void				GetWorldViewProjMatrix(D3DXMATRIX * matWVP)  = 0;
	virtual void				GetClipVolume(CLIPVOLUME & cv) = 0;

	virtual HRESULT	LoadPath(LPCSTR szPath) = 0;
	virtual HRESULT StartPath() = 0;
	virtual HRESULT PausePath() = 0;
	virtual HRESULT StopPath() = 0;// reset
	virtual HRESULT SetSpeed(float fRelativeSpeed) = 0;	

	virtual void	SetFlag(DWORD dwFlag) = 0;
	virtual void	UnsetFlag(DWORD dwFlag) = 0;
	virtual void	InvertFlag(DWORD dwFlag) = 0;
	virtual void	ResetFlags() = 0;


public:	
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;

};