#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "xtexture.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"

XResourceManager::XResourceManager(void)
{
	m_dwRef = 0;
}

XResourceManager::~XResourceManager(void)
{
	
}

HRESULT		XResourceManager::AddRef()
{
	return m_dwRef++;
}

HRESULT		XResourceManager::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XResourceManager object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

HRESULT	XResourceManager::AddSprite(ID3DXSprite * pSpriteToAdd)
{
	if(pSpriteToAdd) {
		m_arSprites.push_back(pSpriteToAdd);		
		return 0;
	}
	return -1;
}



IResource*	XResourceManager::AddResource(IResource *pRes, BOOL bUnique)
{
	DWORD	dwID = 0;int i;
	BOOL bShared = FALSE;
	if(pRes)
	{
		if(!bUnique)
		{
			// Try if the resource is not shared
			for(i = 0; i < (int)m_arResources.size(); i++) {
				if(*m_arResources[i] == *pRes)
				{				
					bShared = TRUE;
					break;				
				}						
			}	
		}
		// Insert resource pointer
		m_arResources.push_back(pRes);
		// Set resource ID
		pRes->SetID(m_arResources.size());
		// If the texture is shared, return parent texture
		if(bShared)
		{
			return m_arResources[i];
		}		
	}
	return NULL;
}

IResource*	 XResourceManager::	GetResource(DWORD dwID)
{
	for(int i = 0; i < (int)m_arResources.size(); i++) {
		if(m_arResources[i]->GetID() == dwID)
		{
			return m_arResources[i];
		}
	}
	return NULL;
}

HRESULT		XResourceManager::AddIndexBuffer(IIndexBuffer * pBufferToAdd)
{
	if(pBufferToAdd) {
		m_arIB.push_back(pBufferToAdd);
		pBufferToAdd->SetID(m_arIB.size());
		return 0;
	}
	return -1;
}

HRESULT		XResourceManager::AddVertexBuffer(IVertexBuffer * pBufferToAdd)
{
	if(pBufferToAdd) {
		m_arVB.push_back(pBufferToAdd);
		pBufferToAdd->SetID(m_arVB.size());
		return 0;
	}
	return -1;
}


HRESULT		XResourceManager::ReleaseAll()
{
	for(int i = 0; i < (int)m_arResources.size(); i++) {
		m_arResources[i]->ReleaseResource();		
	}
	// release textures
	
	for(i = 0; i < (int)m_arVB.size(); i++) {
		m_arVB[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arIB.size(); i++) {
		m_arIB[i]->ReleaseResource();		
	}
	for(i = 0; i < (int)m_arSprites.size(); i++) {
		m_arSprites[i]->OnLostDevice();
	}	
	return 0;
}


HRESULT		XResourceManager::RestoreAll()
{
	for(int i = 0; i < (int)m_arResources.size(); i++) {
		m_arResources[i]->Restore();		
	}
	// restore textures
	
	for(i = 0; i < (int)m_arVB.size(); i++) {
		m_arVB[i]->Restore();		
	}
	for(i = 0; i < (int)m_arIB.size(); i++) {
		m_arIB[i]->Restore();		
	}
	for(i = 0; i < (int)m_arSprites.size(); i++) {
		m_arSprites[i]->OnResetDevice();
	}	
	return 0;
}
	