#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xVertexbuffer.h"

XVertexBuffer::XVertexBuffer(void)
{
	m_lpDevice = NULL;
	m_pVertexBuffer = NULL;
	m_dwRef = 0;
	m_dwID = -1;
}

XVertexBuffer::~XVertexBuffer(void)
{	
	ReleaseResource();
	m_lpDevice = NULL;
}

HRESULT		XVertexBuffer::AddRef()
{
	return m_dwRef++;
}

HRESULT		XVertexBuffer::Release()
{
	DWORD dwID = m_dwID;
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XVertexBuffer object destroyed. ID=%d", dwID);
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

// Tuto metody vola resource manager
int		XVertexBuffer::ReleaseResource()
{
	SAFE_RELEASE(m_pVertexBuffer);
	return S_OK;
}

int		XVertexBuffer::Create(UINT uiLength, DWORD dwUsage, DWORD dwFVF, D3DPOOL dwPool)
{	
	IDisplay *dis = NULL;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();	
	if(!m_pVertexBuffer) 
	{		
		m_uiLength = uiLength;
		m_dwUsage = dwUsage;
		m_dwFVF = dwFVF;
		m_dwPool = dwPool;

		DWORD dwRet = m_lpDevice->CreateVertexBuffer(uiLength, dwUsage, dwFVF, dwPool, &m_pVertexBuffer);
		if(dwRet != S_OK) {
			XException exp("Cannot create vertex buffer.", dwRet);
			THROW(exp);							
		}
		dis->GetResourceManager()->AddVertexBuffer(this);
		SAFE_RELEASE(dis);
		TRACE("XVertexBuffer object created. ID=%d", m_dwID);
		return 0;
	}
	SAFE_RELEASE(dis);
	return -1;
}

int		XVertexBuffer::Restore()
{
	if(m_lpDevice && !m_pVertexBuffer)
	{
		DWORD dwRet = m_lpDevice->CreateVertexBuffer(m_uiLength, m_dwUsage, m_dwFVF, m_dwPool, &m_pVertexBuffer);
		if(dwRet != S_OK) {
			XException exp("Cannot create vertex buffer.", dwRet);
			THROW(exp);				
		}
		return 0;
	}
	return -1;
}
