#include "StdAfx.h"
#include "../common/common.h"
#include "../display/common.h"
#include "../input/common.h"

#include ".\terrain.h"

CTerrain::CTerrain(void)
{
	m_iTerrainTilesX = 0;
	m_iTerrainTilesY = 0;
	m_iTerrainVerticesX = 0;
	m_iTerrainVerticesY = 0;
	m_iTerrainMethod = TM_HEIGHTMAP;
	m_pTerrainVB = NULL;
	m_pTerrainSurface = NULL;
	m_pHeightMap = NULL;
	m_arTerrain = NULL;
	m_dwFlags = TF_DRAWINFO;//TF_WRITEBUFFERS;
	m_iLODLevel = 1;

}

CTerrain::~CTerrain(void)
{
	int i;
	if(m_arTerrain)
	{
		for(i = 0; i < m_iTerrainVerticesX; i++)
		{
			SAFE_DELETE_ARRAY(m_arTerrain[i]);
		}
		SAFE_DELETE_ARRAY(m_arTerrain);
	}
	SAFE_DELETE_ARRAY(m_arLoc);
}

HRESULT			CTerrain::InternalInit()
{
	DWORD dwRet = S_FALSE;
	IDisplay *pDis;	
	CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);
	//
	// Create font 
	if(S_OK == (dwRet = CreateDisplayObject(DISIID_I3DFont, (void**) &m_pDebugFont)))
	{		
		m_pDebugFont->Load(BL_BITMAP, "fps.tga");		
		m_pDebugFont->Load(BL_SIZES, "fps.dat");		
	}
	else
	{
		TRACE("Cannot create font: %d", dwRet);		
		return dwRet;
	}	
	//
	// Load terrain texture
	dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&m_pTerrainSurface);
	if(dwRet == S_OK)
	{
		dwRet = m_pTerrainSurface->LoadTextureFromFileEx("grass64.bmp", 6, pDis->GetTextureFormat());
		if(dwRet != S_OK)
		{
			XException exp("Cannot load texture for terrain surface!", dwRet);
			THROW(exp);
		} 
	}
		
	// Reinit file for IB
	if(m_dwFlags & TF_WRITEBUFFERS)
	{
		fclose(fopen("ib.txt", "w"));
	}

	// Set fog
	pDis->SetPixelFog(D3DFOG_LINEAR, 0, 150,400,D3DCOLOR_ARGB(255, 47,205,251));
	
	SAFE_RELEASE(pDis);
	
	return dwRet;
}

int		CTerrain::ComputeNormals()
{
	D3DXVECTOR3	v0;
	D3DXVECTOR3	s1, s2, s3, s4, s5, s6;
	D3DXVECTOR3 n1, n2, n3, n4, n5, n6, n0, n;
	BYTE vecMask;
	int  iNormCount;
	int x, y;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{
			iNormCount = 0;
			vecMask = 0;
			n0 = D3DXVECTOR3(0,0,0);
			// Vektor, pro ktery normalu pocitame - stredovy vektor
			v0 = m_arTerrain[x][y].vecPos;
			// Nyni vybereme ty spravne body okolo stredoveho vektoru
			// a rovnou urcime smerove vektory k temto bodum
			if(x < m_iTerrainTilesX)
			{
				s1 = m_arTerrain[x+1][y].vecPos - v0;
				vecMask |= 0x01;
			}
					
			if(y < m_iTerrainTilesY)
			{
				s2 = m_arTerrain[x][y+1].vecPos - v0;				
				vecMask |= 0x02;
			}
			if(x > 0 && y < m_iTerrainTilesY)
			{
				s3 = m_arTerrain[x-1][y+1].vecPos - v0;
				vecMask |= 0x04;
			}	
			if(x > 0)
			{
				s4 = m_arTerrain[x-1][y].vecPos - v0;
				vecMask |= 0x08;
			}	
			if(y > 0)
			{
				s5 = m_arTerrain[x][y-1].vecPos - v0;
				vecMask |= 0x10;
			}
			if(y > 0 && x < m_iTerrainTilesX)
			{
				s6 = m_arTerrain[x+1][y-1].vecPos - v0;
				vecMask |= 0x20;
			} 
			// Spocitame normaly pro sousedni dvojice
			// s1 a s2 normala
			if(vecMask & 0x03)
			{
				D3DXVec3Cross(&n, &s1, &s2);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                
			}
			// s2 a s3 normala
			if(vecMask & 0x06)
			{
				D3DXVec3Cross(&n, &s2, &s3);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                  
			}
			// s3 a s4 normala
			if(vecMask & 0x0C)
			{
				D3DXVec3Cross(&n, &s3, &s4);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                  
			}
			// s4 a s5 normala
			if(vecMask & 0x18)
			{
				D3DXVec3Cross(&n, &s4, &s5);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                  
			}
			// s5 a s6 normala
			if(vecMask & 0x30)
			{
				D3DXVec3Cross(&n, &s5, &s6);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                  
			}
			// s6 a s1 normala
			if(vecMask & 0x41)
			{
				D3DXVec3Cross(&n, &s6, &s1);
				D3DXVec3Normalize(&n, &n);
				// Vysledny vektor n0
				n0 += n;
				// Pocet vektoru pro prumer
                iNormCount++;                  
			}			
			// ulozime vysledny vektor - udelame prumer vsech dilcich normal		
			m_arTerrain[x][y].vecNormal = n0 / float(iNormCount);			
		}
	}	
	return S_OK;
}


int		CTerrain::CreateQuadTree()
{
	DWORD dwRet;
	VERTEX	*pVertices;
	WORD *pIndices;
	FILE *f;
	int i, x, y, l, iLeafCount;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	if(m_iTerrainTilesX >= LEAF_I_SIZE && m_iTerrainTilesY >= LEAF_I_SIZE)
	{
		if(m_dwFlags & TF_WRITEBUFFERS)
		{
			f = fopen("buffers.txt", "w");
		
			fprintf(f, "Vertices: %dx%d\nTiles: %dx%d\n\n", 
				m_iTerrainVerticesX, m_iTerrainVerticesY, 
				m_iTerrainTilesX, m_iTerrainTilesY);
		}
		// Create locations
		int iLocCountX = m_iTerrainTilesX / LOC_SIZE; 
		int iLocCountY = m_iTerrainTilesY / LOC_SIZE;
		m_iLocCount = iLocCountX*iLocCountY;
		m_arLoc = new Location[m_iLocCount];
		if(!m_arLoc)
		{
			XException exp("Terrain is not initialized. Invalid location number.");
			THROW(exp);
		}
		// pocet listu jedne lokace
		iLeafCount = LOC_SIZE / LEAF_I_SIZE * LOC_SIZE / LEAF_I_SIZE;
		// vytvoreni IB bufferu lokace a inicilizace okrajovych bodu
		for(y = i = 0; y < iLocCountY; y++)
		{
			for(x = 0; x < iLocCountX; x++)
			{
				if(S_OK == CreateDisplayObject(DISIID_IIndexBuffer, (void**)&m_arLoc[i].pIB))
				{
					// kazdy list ma XxX policek, kazde ma dva trojuhelniky,
					// ktere se indexuji pomoci 3 indexu
					m_arLoc[i].pIB->Create(iLeafCount*LEAF_I_SIZE*LEAF_I_SIZE*2*3*sizeof(WORD), 
						D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY, D3DFMT_INDEX16,  D3DPOOL_DEFAULT);
					
					// 4 okrajove body lokace pro urceni pozice listu v lokaci
					m_arLoc[i].arBounds[0] = BoundingPoint(x*LOC_SIZE, y*LOC_SIZE);
					m_arLoc[i].arBounds[1] = BoundingPoint((x+1)*LOC_SIZE, y*LOC_SIZE);
					m_arLoc[i].arBounds[2] = BoundingPoint(x*LOC_SIZE, (y+1)*LOC_SIZE);
					m_arLoc[i].arBounds[3] = BoundingPoint((x+1)*LOC_SIZE, (y+1)*LOC_SIZE);
					if(m_dwFlags & TF_WRITEBUFFERS)
					{
						fprintf(f, "%d: %dx%d - %dx%d - %dx%d - %dx%d\n\n", i, m_arLoc[i].arBounds[0].x, 
							m_arLoc[i].arBounds[0].y, m_arLoc[i].arBounds[1].x, m_arLoc[i].arBounds[1].y, 
							m_arLoc[i].arBounds[2].x, m_arLoc[i].arBounds[2].y, m_arLoc[i].arBounds[3].x, 
							m_arLoc[i].arBounds[3].y);
					}

					// dalsi parametry potrebne pro rendering
					m_arLoc[i].m_iMinIndex = -1;
					m_arLoc[i].m_iMaxIndex = 0;
					m_arLoc[i].m_iVisQuads = 0;
					m_arLoc[i].m_wID = i+1;
				}
				i++;
			}
		}
		
		// Create VB for terrain and copy all locations
		if(S_OK == CreateDisplayObject(DISIID_IVertexBuffer, (void**)&m_pTerrainVB))
		{
			VERTEX	*NormalsVertices;
			int n; 
			// create vb for all locations
			// kazda lokace ma XxX policek cili X+1 vertexu na kazde strane
			dwRet = m_pTerrainVB->Create(m_iLocCount*LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX), 
				D3DUSAGE_WRITEONLY, VERTEXFORMAT, D3DPOOL_DEFAULT);
			if(dwRet != S_OK)
			{
				XException exp("Terrain is not initialized. Cannot create terrain vertex buffer.", dwRet);
				THROW(exp);
			}
			// copy vertices the the locations
			for(l = 0; l < m_iLocCount; l++)
			{
				// Create VB for terrain and copy all locations
				if(S_OK == CreateDisplayObject(DISIID_IVertexBuffer, (void**)&m_arLoc[l].pNormalsVB))
				{
					dwRet = m_arLoc[l].pNormalsVB->Create(2*LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX),D3DUSAGE_WRITEONLY,VERTEXFORMAT,D3DPOOL_DEFAULT);
					if(dwRet != S_OK)
					{
						XException exp("Terrain is not initialized. Cannot create terrain vertex buffer.", dwRet);
						THROW(exp);
					}
				}
				n = i = 0;
				// zamknout vertexy pro jednu lokaci
				// lokace ma XxX policek cili X+1 vertexu
				m_pTerrainVB->GetBuffer()->Lock(l*LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX), 
					LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX), (BYTE**)&pVertices, 0);
				m_arLoc[l].pNormalsVB->GetBuffer()->Lock(0,0, (BYTE**) &NormalsVertices, 0);
				for(y = m_arLoc[l].arBounds[0].y; y <= m_arLoc[l].arBounds[3].y; y++)
				{
					for(x = m_arLoc[l].arBounds[0].x; x <= m_arLoc[l].arBounds[3].x; x++)
					{
						NormalsVertices[n + 0] = m_arTerrain[x][y];
						NormalsVertices[n + 0].dwDiffuse = D3DCOLOR_ARGB(0, 255, 255, 255);
						NormalsVertices[n + 1] = m_arTerrain[x][y];
						NormalsVertices[n + 1].dwDiffuse = D3DCOLOR_ARGB(0, 255, 255, 0);
						NormalsVertices[n + 1].vecPos += m_arTerrain[x][y].vecNormal;

						pVertices[i] = m_arTerrain[x][y];
						if(m_dwFlags & TF_WRITEBUFFERS)
						{
							fprintf(f, "%d: %f, %f, %f\n", i+l*LOC_V_SIZE*LOC_V_SIZE, pVertices[i].vecPos.x, 
								pVertices[i].vecPos.y, pVertices[i].vecPos.z);
						}
						i++;
						n += 2;
					}
				}
				m_arLoc[l].pNormalsVB->GetBuffer()->Unlock();
				m_pTerrainVB->GetBuffer()->Unlock();
			}
		}

		m_pRoot = new QTNode;
		// Init ROOT
		m_pRoot->ntType = NODE_TYPE;
		m_pRoot->arBounds[0] = m_arTerrain[0][0].vecPos;
		m_pRoot->arBounds[1] = m_arTerrain[m_iTerrainVerticesX-1][0].vecPos;
		m_pRoot->arBounds[2] = m_arTerrain[0][m_iTerrainVerticesY-1].vecPos;
		m_pRoot->arBounds[3] = m_arTerrain[m_iTerrainVerticesX-1][m_iTerrainVerticesY-1].vecPos;
		// Postavit STROM
		CreateNode(m_pRoot);
		//
		// Fill index buffer and leaves with data
		Restore();

		if(m_dwFlags & TF_WRITEBUFFERS)
		{
			fclose(f);
		}
		
		// horni rozsireni oblohy		
		float dif;
		// Create sky
		dwRet = CreateDisplayObject(DISIID_I3DObject, (void**)&m_pSkyBox);
		if(dwRet == S_OK)
		{
			m_pSkyBox->Init(NULL, 0, (D3DFORMAT)0, 8*sizeof(VERTEX), 10*sizeof(WORD), VB_WRITEONLY|IB_WRITEONLY);
			m_pSkyBox->GetVB()->GetBuffer()->Lock(0,0, (BYTE**) &pVertices, 0);
			for(i = y = 0; y < 2; y++)
			{
				for(x = 0; x < 2; x++)
				{					
					pVertices[i].vecPos.x = float(x * (m_iTerrainTilesX));
					pVertices[i].vecPos.y = float(y * (m_iTerrainTilesY));
					pVertices[i].vecPos.z = float(0.0f);
					pVertices[i].dwDiffuse = D3DCOLOR_ARGB(255, 47,205,251);
					// horni vertex
					pVertices[i].tu1 = (i & 0x1) ? 1.0f : 0.0f;
					pVertices[i].tv1 = ((i & 0x6) == 0x6 || (~i & 0x6) == 0x6) ? 0.0f : 1.0f;					
					i++;					

					dif = (i & 0x2) ? 200.0f : -200.0f;					
					pVertices[i].vecPos.x = float(x * m_iTerrainVerticesX + dif);
					dif = (i & 0x4) ? 200.0f : -200.0f;
					pVertices[i].vecPos.y = float(y * m_iTerrainVerticesY + dif);
					pVertices[i].vecPos.z = float(600.0f);	
					pVertices[i].dwDiffuse = D3DCOLOR_ARGB(255, 11,1,152);
					// horni vertex
					pVertices[i].tu1 = (i & 0x1) ? 1.0f : 0.0f;
					pVertices[i].tv1 = ((i & 0x6) == 0x6 || (~i & 0x6) == 0x6) ? 0.0f : 1.0f;					
					i++;					
				}
			}
			m_pSkyBox->GetVB()->GetBuffer()->Unlock();

			m_pSkyBox->GetIB()->GetBuffer()->Lock(0,0, (BYTE**) &pIndices, 0);
			pIndices[0] = 0;pIndices[1] = 1;
			pIndices[2] = 2;pIndices[3] = 3;
			pIndices[4] = 6;pIndices[5] = 7;
			pIndices[6] = 4;pIndices[7] = 5;
			pIndices[8] = 0;pIndices[9] = 1;			
			m_pSkyBox->GetIB()->GetBuffer()->Unlock();			
		}

		dwRet = CreateDisplayObject(DISIID_I3DObject, (void**)&m_pBottom);
		if(dwRet == S_OK)
		{
			m_pBottom->Init(NULL, 0, (D3DFORMAT)0, 4*sizeof(VERTEX), 6*sizeof(WORD), VB_WRITEONLY|IB_WRITEONLY);
			m_pBottom->GetVB()->GetBuffer()->Lock(0,0, (BYTE**) &pVertices, 0);
			pVertices[0].vecPos.x = 0.0;
			pVertices[0].vecPos.y = 0.0;
			pVertices[0].vecPos.z = 0.0;
			pVertices[0].dwDiffuse = D3DCOLOR_ARGB(255, 100, 150, 100);

			pVertices[1].vecPos.x = (float)m_iTerrainTilesX;
			pVertices[1].vecPos.y = 0.0;
			pVertices[1].vecPos.z = 0.0;
			pVertices[1].dwDiffuse = D3DCOLOR_ARGB(255, 100, 150, 100);

			pVertices[2].vecPos.x = 0.0;
			pVertices[2].vecPos.y = (float)m_iTerrainTilesY;
			pVertices[2].vecPos.z = 0.0;
			pVertices[2].dwDiffuse = D3DCOLOR_ARGB(255, 100, 150, 100);

			pVertices[3].vecPos.x = (float)m_iTerrainTilesX;
			pVertices[3].vecPos.y = (float)m_iTerrainTilesY;
			pVertices[3].vecPos.z = 0.0;
			pVertices[3].dwDiffuse = D3DCOLOR_ARGB(255, 100, 150, 100);
			m_pBottom->GetVB()->GetBuffer()->Unlock();

			m_pBottom->GetIB()->GetBuffer()->Lock(0,0, (BYTE**) &pIndices, 0);
			pIndices[0] = 0;
			pIndices[1] = 1;
			pIndices[2] = 2;
			pIndices[3] = 1;
			pIndices[4] = 3;
			pIndices[5] = 2;					
			m_pBottom->GetIB()->GetBuffer()->Unlock();			
		}
	}
	else
	{
		XException exp("Terrain is not initialized. Terrain is too small.");
		THROW(exp);
	}
	return S_OK;
}

HRESULT		CTerrain::GenerateTerrain(int x, int y, TERRAIN_METHOD tmMethod /*= TM_CIRCLE*/)
{
	DWORD dwRet = S_FALSE;
	int i = 0;
	//
	// Init common object (texture, font etc.)
	InternalInit();

	m_iTerrainTilesX = x;
	m_iTerrainTilesY = y;
	m_iTerrainVerticesX = x + 1;
	m_iTerrainVerticesY = y + 1;
	
	m_arTerrain = new VERTEX*[m_iTerrainVerticesX];
	for(i = 0; i < m_iTerrainVerticesX; i++)
	{
		m_arTerrain[i] = new VERTEX[m_iTerrainVerticesY];
	}
	//
	// Init basic terrain parameters
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{				
			m_arTerrain[x][y].vecPos = D3DXVECTOR3(float(x), float(y), 0.0f);						
			m_arTerrain[x][y].dwDiffuse = D3DCOLOR_ARGB(128,128,255,128);
			m_arTerrain[x][y].tu1 = (x % 2) ? 1.0f : 0.0f;
			m_arTerrain[x][y].tv1 = (y % 2) ? 1.0f : 0.0f;	
			m_arTerrain[x][y].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);			
		}
	}
	//
	// Apply circle algorithm on the terrain
	CircleAlgorithm();
	ComputeNormals();
	//
	// Create quad tree
	CreateQuadTree();
	
	return dwRet;
}

HRESULT		CTerrain::GenerateTerrainFromFile(LPCSTR szFile)
{
	DWORD dwRet = S_FALSE;
	int i = 0, x, y;	
	D3DLOCKED_RECT lr;
	int p;
	TEXTURE_PIXEL * data;
	TEXTURE_PIXEL tp;

	//
	// Init common object (texture, font etc.)
	InternalInit();
	//
	// Load heightmap texture
	dwRet = CreateDisplayObject(DISIID_ITexture, (void**)&m_pHeightMap);
	if(dwRet == S_OK)
	{
		dwRet = m_pHeightMap->LoadTextureFromFile(szFile);
		if(dwRet != S_OK)
		{
			XException exp("Cannot load texture for heightmap!", dwRet);
			THROW(exp);
		}                
		if(m_pHeightMap->GetFormat() != D3DFMT_A8R8G8B8 && m_pHeightMap->GetFormat() != D3DFMT_X8R8G8B8)
		{
			XException exp("Invalid heightmap format! Must be 32-bit with alpha channel.");
			THROW(exp);
		}
		//set dim of the terrain according dim of the height map
		m_iTerrainTilesX = m_pHeightMap->Width();
		m_iTerrainTilesY = m_pHeightMap->Height();
		m_iTerrainVerticesX = m_pHeightMap->Width() + 1;
		m_iTerrainVerticesY = m_pHeightMap->Height() + 1;
	}				
	m_arTerrain = new VERTEX*[m_iTerrainVerticesX];
	for(i = 0; i < m_iTerrainVerticesX; i++)
	{
		m_arTerrain[i] = new VERTEX[m_iTerrainVerticesY];
	}
	//
	// Open heightmap
	dwRet = m_pHeightMap->GetTexture()->LockRect(0, &lr, NULL, D3DLOCK_READONLY);	
	data = (TEXTURE_PIXEL*)lr.pBits;
	if(data == NULL)
	{
		XException exp("Cannot lock terrain texture. Data is NULL.", dwRet);
		THROW(exp);
	}		
	// Offset
	p = lr.Pitch/4;
	// Init basic terrain parameters
	for(y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(x = 0; x < m_iTerrainVerticesX; x++)
		{		
			if(y < m_iTerrainVerticesY-1 && x < m_iTerrainVerticesX-1)
			{
				tp = data[y*p + x];
			}
			
			m_arTerrain[x][y].vecPos = D3DXVECTOR3(float(x), float(y), float(tp.a)/255.0f*30.0f);						
			m_arTerrain[x][y].dwDiffuse = D3DCOLOR_ARGB(128, tp.r, tp.g, tp.b);
			m_arTerrain[x][y].tu1 = (x % 2) ? 1.0f : 0.0f;
			m_arTerrain[x][y].tv1 = (y % 2) ? 1.0f : 0.0f;	
			m_arTerrain[x][y].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);

			if(x == 0 || x == m_iTerrainVerticesX-1)
			{
				m_arTerrain[x][y].vecPos.z = 0.0f;
				m_arTerrain[x][y].dwDiffuse = 0;

			}
			if(y == 0 || y == m_iTerrainVerticesY-1)
			{
				m_arTerrain[x][y].vecPos.z = 0.0f;
				m_arTerrain[x][y].dwDiffuse = 0;
			}			
		}
	}
	dwRet = m_pHeightMap->GetTexture()->UnlockRect(0);
	if(dwRet != S_OK)
	{
		XException exp("Cannot unlock texture heightmap!", dwRet);
		THROW(exp);
	}  	
	// Compute normals for all vertices
	ComputeNormals();
	//
	// Create quad tree
	CreateQuadTree();

	return dwRet;
}

// Fills queue of the visible quads
HRESULT		CTerrain::CullTerrain(QTNode *pNode, CLIPVOLUME& cv)
{
	DWORD zones[4] = {0, 0, 0, 0};
	int x, y;		
	float temp, z;
	int u = 0;
	
	for(int i = 0; i < 4; i++)
	{
		x = pNode->arBounds[i].x;
		y = pNode->arBounds[i].y;
		z = pNode->arBounds[i].z;

		temp = cv.pNear.a * x + cv.pNear.b * y + cv.pNear.c * z + cv.pNear.d;
		if (temp > CULL_TOLERANCE) {
			zones[i] |= 0x01;			
		}
		else {
			temp = cv.pFar.a * x + cv.pFar.b * y + cv.pFar.c * z + cv.pFar.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x02;
			}
		}

		temp = cv.pLeft.a * x + cv.pLeft.b * y + cv.pLeft.c * z + cv.pLeft.d;
		if (temp > CULL_TOLERANCE) {
			zones[i] |= 0x04;
		}
		else {
			temp = cv.pRight.a * x + cv.pRight.b * y + cv.pRight.c * z + cv.pRight.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x08;
			}
		}

		temp = cv.pTop.a * x + cv.pTop.b * y + cv.pTop.c * z + cv.pTop.d;
		if (temp > CULL_TOLERANCE+2.0f) {
			zones[i] |= 0x10;
		}
		else 
		{
			temp = cv.pBottom.a * x + cv.pBottom.b * y + cv.pBottom.c * z + cv.pBottom.d;
			if (temp > CULL_TOLERANCE) {
				zones[i] |= 0x20;
			}
		}
	}

	//if all of the corners are outside of the boundaries
	// this node is excluded, so stop traversing
	DWORD res = zones[0] & zones[1] & zones[2] & zones[3];
	if(res)
	{
		return -1;
	}

	// if this is a leaf add the triangle lists to the render queue
	if (pNode->ntType == LEAF_TYPE)
	{	
		IDisplay *pDis;
		D3DXVECTOR3 v, v1, v2;
		float d = 0.0f;		

		// Vzdalenost listu od kamery - pocita se pouze pro LOD urovne > 1
		if(m_iLODLevel != 0)		
		{
			CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);
			v1 = D3DXVECTOR3((float)pNode->arBounds[3].x, (float)pNode->arBounds[3].y, pNode->arBounds[3].z);
			v2 = *pDis->GetCamera()->GetEyePoint();		
			d = D3DXVec3Length(&(v1 - v2));
			// Faktorem list umele priblizime, pokud je faktor vysoky
			// znamena to, ze list je hodne clenity a tudiz nelze
			// nahradit jednodussim provedenim a proto ho umele priblizime
			// a tim zpusobime, ze se nenahradi
			d -= pNode->fLODFac * (MAX_LOD_LEVEL-m_iLODLevel);	
			SAFE_RELEASE(pDis);
		}		

		// Maximalni a minimalni index v ramci lokace
		// Nalezeni minimalniho a maximalniho indexu pro danou lokaci
		// Tyto hodnoty se upravuji behem rozpoznavani viditelnych listu
		// a vyuziji se pri renderingu
		if(pNode->pLocation->m_iMinIndex > pNode->pIndices1[0])
		{
			pNode->pLocation->m_iMinIndex = pNode->pIndices1[0];
		}
		if(pNode->pLocation->m_iMaxIndex < pNode->pIndices1[pNode->wInd1Count-1])
		{
			pNode->pLocation->m_iMaxIndex = pNode->pIndices1[pNode->wInd1Count-1];
		}	

		if(d < 150.0f)
		{
			int lw = pNode->pLocation->m_iVisTiles * 6;
			// Zkopirovani vsech indexu listu do spravne pozice IB			
			memcpy(pNode->pLocation->pIndices+lw, pNode->pIndices1, pNode->wInd1Count*sizeof(WORD));
			// Zvysit pocet viditelnych listu pro danou lokaci
			// Opet se vyuzije pri renderingu (pocita se pocet primitiv)
			pNode->pLocation->m_iVisQuads++;
			pNode->pLocation->m_iVisTiles += pNode->wTiles1Count;			
		}
		else
		{
			if(d >= 150.0f && d < 200.f)
			{
				int lw = pNode->pLocation->m_iVisTiles * 6;
				// Zkopirovani vsech indexu listu do spravne pozice IB				
				memcpy(pNode->pLocation->pIndices+lw, pNode->pIndices2, pNode->wInd2Count*sizeof(WORD));				
				// Zvysit pocet viditelnych listu pro danou lokaci
				// Opet se vyuzije pri renderingu (pocita se pocet primitiv)
				pNode->pLocation->m_iVisQuads++;
				pNode->pLocation->m_iVisTiles += pNode->wTiles2Count;				
			}
			else
			{
				if(d >= 200.0f && d < 250)
				{
					int lw = pNode->pLocation->m_iVisTiles * 6;
					// Zkopirovani vsech indexu listu do spravne pozice IB
					memcpy(pNode->pLocation->pIndices+lw, pNode->pIndices3, pNode->wInd3Count*sizeof(WORD));				
					// Zvysit pocet viditelnych listu pro danou lokaci
					// Opet se vyuzije pri renderingu (pocita se pocet primitiv)
					pNode->pLocation->m_iVisQuads++;
					pNode->pLocation->m_iVisTiles += pNode->wTiles3Count;
					
				}
				else
				{
					if(d >= 250.0f && d < 300)
					{
						int lw = pNode->pLocation->m_iVisTiles * 6;
						// Zkopirovani vsech indexu listu do spravne pozice IB
						memcpy(pNode->pLocation->pIndices+lw, pNode->pIndices4, pNode->wInd4Count*sizeof(WORD));					
						// Zvysit pocet viditelnych listu pro danou lokaci
						// Opet se vyuzije pri renderingu (pocita se pocet primitiv)
						pNode->pLocation->m_iVisQuads++;
						pNode->pLocation->m_iVisTiles += pNode->wTiles4Count;						
					}
					else
					{
						int lw = pNode->pLocation->m_iVisTiles * 6;
						// Zkopirovani vsech indexu listu do spravne pozice IB
						memcpy(pNode->pLocation->pIndices+lw, pNode->pIndices5, pNode->wInd5Count*sizeof(WORD));						
						// Zvysit pocet viditelnych listu pro danou lokaci
						// Opet se vyuzije pri renderingu (pocita se pocet primitiv)
						pNode->pLocation->m_iVisQuads++;
						pNode->pLocation->m_iVisTiles += pNode->wTiles5Count;						
					}
				}
			}
		}
		// Nastaveni ze tento list je videt (pouze pro vykresleni mapy)
		pNode->bVis = TRUE;	
		return 1;
	}
	else 
	{
		//this is not a leaf traverse deeper
		for(i = 0; i < 4; i++) {
			CullTerrain(pNode->pBranches[i], cv);
		}		
	}
	return 0;
}


HRESULT		CTerrain::Render()
{
	IDisplay *pDis;
	CLIPVOLUME cv;
	PIXEL v;
	int i, l;
	int iVisLoc = 0;
	int iVisLeaves = 0;
	int iVisFaces = 0;
	char szInfo[128];
	CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);
	
	// Pokud se pouziva mapa, je treba vynulovat viditelnosti
	// vsech listu
	if(m_dwFlags & TF_DRAWQUADSMAP)
	{
		for(i = 0; i < (int)m_arLeaves.size(); i++)
		{
			((QTNode*)m_arLeaves[i])->bVis = FALSE;
		}
	}
	
	// Volba zaruci ze se prestane rozpoznavat
	// viditelnosti listu a pocitani ostatnich
	// parametru nutnych pro vykresleni
	// pouziji se hodnoty z predchozich framu
	if(!(m_dwFlags & TF_FROZECULLING))
	{
		// Pro kazdou lokaci odemkni INDEX buffer 
		// a priprav dalsi promenne potrebne pro rendering		
		for(l = 0; l < m_iLocCount; l++)
		{
			m_arLoc[l].pIB->GetBuffer()->Lock(0,0,(BYTE**)&m_arLoc[l].pIndices, D3DLOCK_DISCARD);
			m_arLoc[l].m_iVisQuads = 0;
			m_arLoc[l].m_iVisTiles = 0;
			m_arLoc[l].m_iMinIndex = -1;// tady to pretece
			m_arLoc[l].m_iMaxIndex = 0;
		}		
		
		// Vlasti culling, pouzije se CV z kamery a projde se cely strom
		pDis->GetCamera()->GetClipVolume(cv);
		BEGIN_PROFILER();
		CullTerrain(m_pRoot, cv);				
		END_PROFILER("CullTerrain()");
		// Opetovne zamknuti IBs opet pro kazdou lokaci.
		for(l = 0; l < m_iLocCount; l++)
		{
			m_arLoc[l].pIB->GetBuffer()->Unlock();
		}		
	}

	BEGIN_PROFILER();
	pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
	pDis->EnableFog(FALSE);	
	m_pSkyBox->Draw(D3DPT_TRIANGLESTRIP, 0, 8, 0, 8);	
	
	pDis->EnableFog(TRUE);
	m_pBottom->Draw(D3DPT_TRIANGLELIST, 0, 4, 0, 2);
	
	pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	// nastaveni standarnich parametru nutnych pro vykrelseni
	// VERTEX shader (jako fixed)
	// Jeden hlavni vertex buffer a textura travy
	pDis->GetDevice()->SetVertexShader(VERTEXFORMAT);
	pDis->GetDevice()->SetStreamSource(0, m_pTerrainVB->GetBuffer(), sizeof(VERTEX));
	pDis->GetDevice()->SetTexture(0, m_pTerrainSurface->GetTexture());
	END_PROFILER("Prepare render");
	// Nyni se vykresli vsechny viditelne listy po lokacich
	BEGIN_PROFILER();
	for(l = m_iLocCount-1; l >= 0; l--)
	{			
		// Ma smysl vykreslovat pouze kdyz je viditelny nejaky list
		if(m_arLoc[l].m_iVisTiles > 0)
		{			
			// Nastaveni IB pro danou lokaci a posunuti indexu ve VB
			pDis->GetDevice()->SetIndices(m_arLoc[l].pIB->GetBuffer(), l*LOC_V_SIZE*LOC_V_SIZE);
			// Vlastni rendering vyuziva parametry (viz. vyse) z funkce CullTerrain()
			pDis->GetDevice()->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, m_arLoc[l].m_iMinIndex, 
											m_arLoc[l].m_iMaxIndex-m_arLoc[l].m_iMinIndex+1, 0, 
											m_arLoc[l].m_iVisTiles*2);
			
			// Debug informace
			iVisLeaves += m_arLoc[l].m_iVisQuads;
			iVisFaces += m_arLoc[l].m_iVisTiles*2;
			iVisLoc++;			
		}			
	}	
	END_PROFILER("RenderLoc");	

	if(m_dwFlags & TF_SHOWNORMALS)
	{
		pDis->GetDevice()->SetVertexShader(VERTEXFORMAT);
		pDis->GetDevice()->SetTexture(0, NULL);
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
		for(l = m_iLocCount-1; l >= 0; l--)
		{	
			if(m_arLoc[l].m_iVisTiles > 0)
			{
				pDis->GetDevice()->SetStreamSource(0, m_arLoc[l].pNormalsVB->GetBuffer(), sizeof(VERTEX));
				pDis->GetDevice()->DrawPrimitive(D3DPT_LINELIST, 0, LOC_V_SIZE*LOC_V_SIZE);
			}
		}
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	}
	// Debug informace
	sprintf(szInfo, "Using quad tree culling\nTotal leaves: %d\nTotal faces: %d\nLocations: %d\nLOD Level #%d", iVisLeaves, iVisFaces, iVisLoc, m_iLODLevel);

	// Volba umoznuje vykresleni mapy viditelnych listu v pravem rohu obrazovky	
	if(m_dwFlags & TF_DRAWQUADSMAP)
	{
		pDis->GetDevice()->SetVertexShader(PIXELFORMAT);			
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
		pDis->GetDevice()->SetTexture(0, NULL);
		for(i = 0; i < (int)m_arLeaves.size(); i++)
		{
			QTNode *pNode = (QTNode*)m_arLeaves[i];
			v.vecPos.x = pNode->arBounds[0].x/LEAF_I_SIZE + pDis->GetResolution()->x - m_iTerrainTilesX/LEAF_I_SIZE - 10;
			v.vecPos.y = float(pNode->arBounds[0].y)/LEAF_I_SIZE + 10;
			v.vecPos.z = 0.0f;
			v.rhw = 1.0f;
			if(pNode->bVis)
			{
				v.dwDiffuse = D3DCOLOR_ARGB(255,255,255,0);				
			}
			else
			{
				v.dwDiffuse = D3DCOLOR_ARGB(255,128,128,128);
			}
			pDis->GetDevice()->DrawPrimitiveUP(D3DPT_POINTLIST, 1, &v, sizeof(PIXEL));			
		}
		pDis->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
	}	
	if(TF_DRAWINFO & m_dwFlags)
	{
		// Debug informace	
		BEGIN_PROFILER();
		m_pDebugFont->DrawText(szInfo, 0, 460, D3DCOLOR_ARGB(100,150,255,100));	
		END_PROFILER("Draw terrain info.");
	}
	SAFE_RELEASE(pDis);
	return 0;
}

HRESULT		CTerrain::Restore()
{
	DWORD dwRet = S_FALSE;
	VERTEX* pVertices;
	int l, x, y, i;
	if(!m_arTerrain && m_pTerrainVB)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	// copy vertices the the locations
	for(l = 0; l < m_iLocCount; l++)
	{
		i = 0;
		// zamknout vertexy pro jednu lokaci
		// lokace ma XxX policek cili X+1 vertexu
		m_pTerrainVB->GetBuffer()->Lock(l*LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX), LOC_V_SIZE*LOC_V_SIZE*sizeof(VERTEX), (BYTE**)&pVertices, 0);
		for(y = m_arLoc[l].arBounds[0].y; y <= m_arLoc[l].arBounds[3].y; y++)
		{
			for(x = m_arLoc[l].arBounds[0].x; x <= m_arLoc[l].arBounds[3].x; x++)
			{
				pVertices[i] = m_arTerrain[x][y];				
				i++;
			}
		}
		m_pTerrainVB->GetBuffer()->Unlock();
	}	
	return dwRet;
}

HRESULT		CTerrain::DestroyTerrain()
{
	DWORD dwRet = S_FALSE;

	DestroyNode(m_pRoot);
	for(int i = 0; i < m_iLocCount; i++)
	{		
		SAFE_RELEASE(m_arLoc[i].pNormalsVB);
		SAFE_RELEASE(m_arLoc[i].pIB);
	}	
	SAFE_RELEASE(m_pHeightMap);
	SAFE_RELEASE(m_pTerrainSurface);	
	SAFE_RELEASE(m_pDebugFont);
	SAFE_RELEASE(m_pTerrainVB);
	SAFE_RELEASE(m_pSkyBox);
	SAFE_RELEASE(m_pBottom);

	return dwRet;
}

int		CTerrain::CreateNode(QTNode *pNode)
{
	UINT		uiWidth, uiHeight;
	int			x, y, i, i2, i3, i4, i5;
	FILE *f;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	if(pNode)
	{			
		pNode->pLocation = NULL;
		//
		// Compute node width and height
		uiWidth =  int(pNode->arBounds[1].x - pNode->arBounds[0].x);
		uiHeight = int(pNode->arBounds[2].y - pNode->arBounds[0].y);
		//
		// Get node type
		if(uiWidth == LEAF_I_SIZE)
		{
			if(m_dwFlags & TF_WRITEBUFFERS)
			{
				f = fopen("ib.txt", "a");
			}
			pNode->ntType = LEAF_TYPE;
			// compute middle of the leaf
			x = pNode->arBounds[0].x + uiWidth/2;
			y = pNode->arBounds[0].y + uiHeight/2;
			// get location that is around the middle point
			for(i = 0; i < m_iLocCount; i++)
			{
				if( m_arLoc[i].arBounds[0].x <= x &&
					m_arLoc[i].arBounds[0].y <= y &&
					m_arLoc[i].arBounds[3].x >= x &&
					m_arLoc[i].arBounds[3].y >= y)
				{
					pNode->pLocation = &m_arLoc[i];					
					if(m_dwFlags & TF_WRITEBUFFERS)
					{
						fprintf(f, "\nNode %d\nLocation %d\n->\n", m_arLeaves.size(), pNode->pLocation->m_wID);
					}
					break;
				}
			}

			if(m_dwFlags & TF_WRITEBUFFERS)
			{
				fprintf(f, "\n\nLOD #1:\n");
			}
			
			pNode->bVis = FALSE;			
			m_arLeaves.push_back(pNode);
			// allocate space for indices
			pNode->wInd1Count = LEAF_I_SIZE*LEAF_I_SIZE*2*3;
			pNode->pIndices1 = new WORD[pNode->wInd1Count];
			pNode->wTiles1Count = LEAF_I_SIZE*LEAF_I_SIZE;
			// allocate space for indices
			pNode->wInd2Count = LEAF_I_SIZE/2*LEAF_I_SIZE/2*2*3;
			pNode->pIndices2 = new WORD[pNode->wInd2Count];
			pNode->wTiles2Count = LEAF_I_SIZE/2*LEAF_I_SIZE/2;
			// allocate space for indices
			pNode->wInd3Count = LEAF_I_SIZE/4*LEAF_I_SIZE/4*2*3;
			pNode->pIndices3 = new WORD[pNode->wInd3Count];
			pNode->wTiles3Count = LEAF_I_SIZE/4*LEAF_I_SIZE/4;
			// allocate space for indices
			pNode->wInd4Count = LEAF_I_SIZE/8*LEAF_I_SIZE/8*2*3;
			pNode->pIndices4 = new WORD[pNode->wInd4Count];
			pNode->wTiles4Count = LEAF_I_SIZE/8*LEAF_I_SIZE/8;
			// allocate space for indices
			pNode->wInd5Count = LEAF_I_SIZE/16*LEAF_I_SIZE/16*2*3;
			pNode->pIndices5 = new WORD[pNode->wInd5Count];
			pNode->wTiles5Count = LEAF_I_SIZE/16*LEAF_I_SIZE/16;

			i3 =i2 =i4 = i5 = i = 0;
			float MaxZ = -255.0, MinZ = 255.0;
			 
			int by, bx;
			for(y = pNode->arBounds[0].y; y < pNode->arBounds[3].y; y++)
			{
				for(x = pNode->arBounds[0].x; x < pNode->arBounds[3].x; x++)
				{	
					if(m_arTerrain[x][y].vecPos.z > MaxZ)
					{
						MaxZ = m_arTerrain[x][y].vecPos.z;
					}
					if(m_arTerrain[x][y].vecPos.z < MinZ)
					{
						MinZ = m_arTerrain[x][y].vecPos.z;
					}
					
					bx = x % LOC_SIZE;
					by = y % LOC_SIZE;					

					
					pNode->pIndices1[i + 0] = bx + by * LOC_V_SIZE;
					pNode->pIndices1[i + 1] = (bx+1) + by * LOC_V_SIZE;
					pNode->pIndices1[i + 2] = bx + (by+1) * LOC_V_SIZE;	

					if(x % 2 == 0 && y % 2 == 0)
					{						
						pNode->pIndices2[i2 + 0] = bx + by * LOC_V_SIZE;
						pNode->pIndices2[i2 + 1] = (bx+2) + by * LOC_V_SIZE;
						pNode->pIndices2[i2 + 2] = bx + (by+2) * LOC_V_SIZE;
						
						i2 += 3;
					}
					if(x % 4 == 0 && y % 4 == 0)
					{
						pNode->pIndices3[i3 + 0] = bx + by * LOC_V_SIZE;
						pNode->pIndices3[i3 + 1] = (bx+4) + by * LOC_V_SIZE;
						pNode->pIndices3[i3 + 2] = bx + (by+4) * LOC_V_SIZE;	
						i3 += 3;
					}
					if(x % 8 == 0 && y % 8 == 0)
					{
						pNode->pIndices4[i4 + 0] = bx + by * LOC_V_SIZE;
						pNode->pIndices4[i4 + 1] = (bx+8) + by * LOC_V_SIZE;
						pNode->pIndices4[i4 + 2] = bx + (by+8) * LOC_V_SIZE;	
						i4 += 3;
					}
					if(x % 16 == 0 && y % 16 == 0)
					{
						pNode->pIndices5[i5 + 0] = bx + by * LOC_V_SIZE;
						pNode->pIndices5[i5 + 1] = (bx+16) + by * LOC_V_SIZE;
						pNode->pIndices5[i5 + 2] = bx + (by+16) * LOC_V_SIZE;	
						i5 += 3;
					}

					if(m_dwFlags & TF_WRITEBUFFERS)
					{
						fprintf(f, "%d: %d, %d, %d\n", i, pNode->pIndices1[i + 0], pNode->pIndices1[i + 1], pNode->pIndices1[i + 2]);
					}			
					
					i += 3;

					pNode->pIndices1[i + 0] = (bx+1) + by * LOC_V_SIZE;
					pNode->pIndices1[i + 1] = (bx+1) + (by+1) * LOC_V_SIZE;
					pNode->pIndices1[i + 2] = bx + (by+1) * LOC_V_SIZE;	

					if(x % 2 == 0 && y % 2 == 0)
					{						
						pNode->pIndices2[i2 + 0] = (bx+2) + by * LOC_V_SIZE;
						pNode->pIndices2[i2 + 1] = (bx+2) + (by+2) * LOC_V_SIZE;
						pNode->pIndices2[i2 + 2] = bx + (by+2) * LOC_V_SIZE;
						if(m_dwFlags & TF_WRITEBUFFERS)
						{
							fprintf(f, "%d: %d, %d, %d\n", i2, pNode->pIndices2[i2 + 0], pNode->pIndices2[i2 + 1], pNode->pIndices2[i2 + 2]);
						}
						i2 += 3;
					}
					if(x % 4 == 0 && y % 4 == 0)
					{
						pNode->pIndices3[i3 + 0] = (bx+4) + by * LOC_V_SIZE;
						pNode->pIndices3[i3 + 1] = (bx+4) + (by+4) * LOC_V_SIZE;
						pNode->pIndices3[i3 + 2] = bx + (by+4) * LOC_V_SIZE;	
						i3 += 3;
					}
					if(x % 8 == 0 && y % 8 == 0)
					{
						pNode->pIndices4[i4 + 0] = (bx+8) + by * LOC_V_SIZE;
						pNode->pIndices4[i4 + 1] = (bx+8) + (by+8) * LOC_V_SIZE;
						pNode->pIndices4[i4 + 2] = bx + (by+8) * LOC_V_SIZE;	
						i4 += 3;
					}
					if(x % 16 == 0 && y % 16 == 0)
					{
						pNode->pIndices5[i5 + 0] = (bx+16) + by * LOC_V_SIZE;
						pNode->pIndices5[i5 + 1] = (bx+16) + (by+16) * LOC_V_SIZE;
						pNode->pIndices5[i5 + 2] = bx + (by+16) * LOC_V_SIZE;	
						i5 += 3;
					}
					if(m_dwFlags & TF_WRITEBUFFERS)
					{
						fprintf(f, "%d: %d, %d, %d\n", i,  pNode->pIndices1[i + 0], pNode->pIndices1[i + 1], pNode->pIndices1[i + 2]);
					}
					i += 3;	
				}
			}

			pNode->fLODFac = MaxZ - MinZ;

			if(m_dwFlags & TF_WRITEBUFFERS)
			{
				fprintf(f, "\n\nLOD #2:\n");
			}			

			if(m_dwFlags & TF_WRITEBUFFERS)
			{
				fprintf(f, "Leaf end\n");
				fclose(f);
			}
			return 1;
		}
		else
		{
			pNode->ntType = NODE_TYPE;
			//
			// Create 4 children

			// First
			pNode->pBranches[0] = new QTNode;
			// Init new bounding box
			pNode->pBranches[0]->arBounds[0] = pNode->arBounds[0];

			pNode->pBranches[0]->arBounds[1].x = pNode->arBounds[1].x - uiWidth / 2;
			pNode->pBranches[0]->arBounds[1].y = pNode->arBounds[1].y;
			pNode->pBranches[0]->arBounds[1].z = m_arTerrain[pNode->pBranches[0]->arBounds[1].x][pNode->pBranches[0]->arBounds[1].y].vecPos.z;

			pNode->pBranches[0]->arBounds[2].x = pNode->arBounds[2].x;
			pNode->pBranches[0]->arBounds[2].y = pNode->arBounds[2].y - uiHeight / 2;
			pNode->pBranches[0]->arBounds[2].z = m_arTerrain[pNode->pBranches[0]->arBounds[2].x][pNode->pBranches[0]->arBounds[2].y].vecPos.z;

			pNode->pBranches[0]->arBounds[3].x = pNode->arBounds[3].x - uiWidth / 2;
			pNode->pBranches[0]->arBounds[3].y = pNode->arBounds[3].y - uiHeight / 2;
			pNode->pBranches[0]->arBounds[3].z = m_arTerrain[pNode->pBranches[0]->arBounds[3].x][pNode->pBranches[0]->arBounds[3].y].vecPos.z;
	
			// Second
			pNode->pBranches[1] = new QTNode;
			// Init new bounding box
			pNode->pBranches[1]->arBounds[0].x = pNode->arBounds[0].x + uiWidth / 2;
			pNode->pBranches[1]->arBounds[0].y = pNode->arBounds[0].y;			
			pNode->pBranches[1]->arBounds[0].z = m_arTerrain[pNode->pBranches[1]->arBounds[0].x][pNode->pBranches[1]->arBounds[0].y].vecPos.z;

			pNode->pBranches[1]->arBounds[1] = pNode->arBounds[1];
			
			pNode->pBranches[1]->arBounds[2].x = pNode->arBounds[2].x + uiWidth / 2;
			pNode->pBranches[1]->arBounds[2].y = pNode->arBounds[2].y - uiHeight / 2;			
			pNode->pBranches[1]->arBounds[2].z = m_arTerrain[pNode->pBranches[1]->arBounds[2].x][pNode->pBranches[1]->arBounds[2].y].vecPos.z;

			pNode->pBranches[1]->arBounds[3].x = pNode->arBounds[3].x;
			pNode->pBranches[1]->arBounds[3].y = pNode->arBounds[3].y - uiHeight / 2;
			pNode->pBranches[1]->arBounds[3].z = m_arTerrain[pNode->pBranches[1]->arBounds[3].x][pNode->pBranches[1]->arBounds[3].y].vecPos.z;

			// Third
			pNode->pBranches[2] = new QTNode;
			// Init new bounding box
			pNode->pBranches[2]->arBounds[0].x = pNode->arBounds[0].x;
			pNode->pBranches[2]->arBounds[0].y = pNode->arBounds[0].y + uiHeight / 2;			
			pNode->pBranches[2]->arBounds[0].z = m_arTerrain[pNode->pBranches[2]->arBounds[0].x][pNode->pBranches[2]->arBounds[0].y].vecPos.z;
			
			pNode->pBranches[2]->arBounds[1].x = pNode->arBounds[1].x - uiWidth / 2;
			pNode->pBranches[2]->arBounds[1].y = pNode->arBounds[1].y + uiHeight / 2;			
			pNode->pBranches[2]->arBounds[1].z = m_arTerrain[pNode->pBranches[2]->arBounds[1].x][pNode->pBranches[2]->arBounds[1].y].vecPos.z;

			pNode->pBranches[2]->arBounds[2] = pNode->arBounds[2];

			pNode->pBranches[2]->arBounds[3].x = pNode->arBounds[3].x - uiWidth / 2;
			pNode->pBranches[2]->arBounds[3].y = pNode->arBounds[3].y;			
			pNode->pBranches[2]->arBounds[3].z = m_arTerrain[pNode->pBranches[2]->arBounds[3].x][pNode->pBranches[2]->arBounds[3].y].vecPos.z;

			// Fourth
			pNode->pBranches[3] = new QTNode;
			// Init new bounding box
			pNode->pBranches[3]->arBounds[0].x = pNode->arBounds[0].x + uiWidth / 2;
			pNode->pBranches[3]->arBounds[0].y = pNode->arBounds[0].y + uiHeight / 2;			
			pNode->pBranches[3]->arBounds[0].z = m_arTerrain[pNode->pBranches[3]->arBounds[0].x][pNode->pBranches[3]->arBounds[0].y].vecPos.z;
			
			pNode->pBranches[3]->arBounds[1].x = pNode->arBounds[1].x;
			pNode->pBranches[3]->arBounds[1].y = pNode->arBounds[1].y + uiHeight / 2;				
			pNode->pBranches[3]->arBounds[1].z = m_arTerrain[pNode->pBranches[3]->arBounds[1].x][pNode->pBranches[3]->arBounds[1].y].vecPos.z;

			pNode->pBranches[3]->arBounds[2].x = pNode->arBounds[2].x + uiWidth / 2;
			pNode->pBranches[3]->arBounds[2].y = pNode->arBounds[2].y;	
			pNode->pBranches[3]->arBounds[2].z = m_arTerrain[pNode->pBranches[2]->arBounds[2].x][pNode->pBranches[3]->arBounds[2].y].vecPos.z;

			pNode->pBranches[3]->arBounds[3] = pNode->arBounds[3];

			// Create children of the children
			CreateNode(pNode->pBranches[0]);
			CreateNode(pNode->pBranches[1]);
			CreateNode(pNode->pBranches[2]);
			CreateNode(pNode->pBranches[3]);
		}
		return 0;
	}
	return -1;
}

int		CTerrain::DestroyNode(QTNode *pNode)
{
	if(pNode)
	{
		if(pNode->ntType == LEAF_TYPE)
		{
			SAFE_DELETE_ARRAY(pNode->pIndices1);
			SAFE_DELETE_ARRAY(pNode->pIndices2);
			SAFE_DELETE_ARRAY(pNode->pIndices3);
			SAFE_DELETE_ARRAY(pNode->pIndices4);
			SAFE_DELETE_ARRAY(pNode->pIndices5);
		}
		else
		{
			DestroyNode(pNode->pBranches[0]);
			DestroyNode(pNode->pBranches[1]);
			DestroyNode(pNode->pBranches[2]);
			DestroyNode(pNode->pBranches[3]);
		}
		SAFE_DELETE(pNode);
		return 0;
	}
	return -1;
}

int	CTerrain::CircleAlgorithm()
{
	int iCircleX, iCircleY;
	char per[50];
	double pd, circlesize, distance, disp, fPer = 0.0f;
	int	iIterCount = cmnReadSetupInt(_S_TERRAIN, _S_ITERCOUNT);
	int iDisplacement = cmnReadSetupInt(_S_TERRAIN, _S_DISPLACEMENT);
	int iMaxHeight = cmnReadSetupInt(_S_TERRAIN, _S_MAXHEIGHT) * 10;
	//
	// Check terrain validity
	if(!m_arTerrain)
	{
		XException exp("Terrain is not initialized. You cannot apply any algorithm.");
		THROW(exp);
	}
	// reset terrain
	for(int y = 0; y < m_iTerrainVerticesY; y++)
	{
		for(int x = 0; x < m_iTerrainVerticesX; x++)
		{
			m_arTerrain[x][y].vecPos.z = -10.0f;
		}
	}
	for(int i = 0; i < iIterCount; i++)
	{
		// choose random circle center
		iCircleX = rand() % m_iTerrainVerticesX;
		iCircleY = rand() % m_iTerrainVerticesY;

		circlesize = (rand() % iMaxHeight) / 10.0f + 10.0f;
		disp = (rand() % iDisplacement) / 10.0f;

		// apply displacement
		for(int y = 0; y < m_iTerrainVerticesY; y++)
		{
			for(int x = 0; x < m_iTerrainVerticesX; x++)
			{
				// compute distance current point and center point
				distance = sqrt(pow(m_arTerrain[iCircleX][iCircleY].vecPos.x - m_arTerrain[x][y].vecPos.x, 2) + pow(m_arTerrain[iCircleX][iCircleY].vecPos.y - m_arTerrain[x][y].vecPos.y, 2)); 

				pd = distance * 2 / circlesize;

				if(fabs(pd) <= 1.0f)
				{
					m_arTerrain[x][y].vecPos.z += float(disp / 2 + cos(pd * D3DX_PI) * disp / 2);
				}
			}
		}
		IDisplay *pDis;
		CreateDisplayObject(DISIID_IDisplay, (void**) &pDis);

		fPer += (100.0f/iIterCount);
		pDis->UpdateBackground();
		pDis->GetDevice()->BeginScene();
		sprintf(per, "Generating terrain: %3.1lf%%", fPer);
		m_pDebugFont->DrawText(per, 0, 50, D3DCOLOR_ARGB(255,255,255,0));
		pDis->GetDevice()->EndScene();
		pDis->Present();

		SAFE_RELEASE(pDis);
	}
	return S_OK;
}
